@@ -876,10 +876,12 @@ int git_describe_options_init(git_describe_options *opts, unsigned int version)
 	return 0;
 }
 
+#ifndef GIT_DEPRECATE_HARD
 int git_describe_init_options(git_describe_options *opts, unsigned int version)
 {
 	return git_describe_options_init(opts, version);
 }
+#endif
 
 int git_describe_format_options_init(git_describe_format_options *opts, unsigned int version)
 {
@@ -888,7 +890,9 @@ int git_describe_format_options_init(git_describe_format_options *opts, unsigned
 	return 0;
 }
 
+#ifndef GIT_DEPRECATE_HARD
 int git_describe_init_format_options(git_describe_format_options *opts, unsigned int version)
 {
 	return git_describe_format_options_init(opts, version);
 }
+#endif
